<?php

declare(strict_types = 1);

namespace Drupal\Tests\views_st\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\views\Entity\View;
use Drupal\views\ViewEntityInterface;

/**
 * @group views_st
 */
class ViewStAdminContentTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'block',
    'views',
    'views_ui',
    'views_st',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'olivero';

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function setUp(): void {
    parent::setUp();

    $this->adminUser = $this->drupalCreateUser([
      // Node.
      'administer nodes',
      'access content overview',
      // System.
      'access administration pages',
      'access content',
      'administer themes',
      'view the administration theme',
      // Views Secondary Tabs.
      'views_st.views_st.admin',
      // Views UI.
      'administer views',
    ]);
    $this->drupalLogin($this->adminUser);

    $this
      ->installTheme('claro')
      ->setAdminTheme('claro');
  }

  /**
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   */
  public function testAdminContent(): void {
    $stParentId = 'system.admin_content';
    $stWeight = 0;

    $this->createViewsSecondaryTab([
      'id' => 'admin_content_node_all',
      'parent[id]' => $stParentId,
      'weight' => $stWeight,
      'view[id_and_display]' => 'content:page_1',
    ]);

    foreach (['article', 'news'] as $nodeTypeId) {
      $this->drupalCreateContentType(['type' => $nodeTypeId]);
      $this->createViewWithPageDisplay(
        'node',
        $nodeTypeId,
        [
          'id' => "admin_content_node_$nodeTypeId",
        ],
      );
      $this->createViewsSecondaryTab([
        'id' => "admin_content_node_$nodeTypeId",
        'parent[id]' => $stParentId,
        'weight' => ++$stWeight,
        'view[id_and_display]' => "admin_content_node_$nodeTypeId:page_1",
      ]);
    }

    drupal_flush_all_caches();
    $this->drupalGet('admin/content');
    $page = $this->getSession()->getPage();
    $secondaryTabElements = $page->findAll('css', '.tabs.tabs--secondary > .tabs__tab > a.tabs__link');

    static::assertCount(
      3,
      $secondaryTabElements,
      'There are three Secondary tabs.',
    );

    $expected = [
      [
        'text' => 'Content(active tab)',
        'href' => '/admin/content',
      ],
      [
        'text' => 'article',
        'href' => '/admin/content/node/article',
      ],
      [
        'text' => 'news',
        'href' => '/admin/content/node/news',
      ],
    ];
    foreach ($secondaryTabElements as $key => $secondaryTabElement) {
      static::assertSame($expected[$key]['text'], $secondaryTabElement->getText());
      static::assertSame($expected[$key]['href'], $secondaryTabElement->getAttribute('href'));
    }
  }

  protected function createViewWithPageDisplay(
    string $entityTypeId,
    string $bundleId,
    array $values,
  ): static {
    $wizardKey = $entityTypeId;
    $pagePath = match ($entityTypeId) {
      'node' => "/admin/content/$entityTypeId/$bundleId",
      default => NULL,
    };

    $formValues = [
      'label' => $bundleId,
      'id' => $values['id'],
      'show[wizard_key]' => $wizardKey,
      'show[type]' => $bundleId,
      'page[create]' => TRUE,
      'page[title]' => $bundleId,
      'page[path]' => $pagePath,
      'page[style][row_plugin]' => 'titles',
    ];

    $this->drupalGet('admin/structure/views/add');
    $this->submitForm($formValues, 'Save and edit');

    $view = View::load($values['id']);
    static::assertInstanceOf(ViewEntityInterface::class, $view);

    return $this;
  }

  /**
   * Creates a new Secondary tab.
   *
   * Required keys:
   * - id
   * - parent[id]
   * - view[id_and_display]
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   */
  protected function createViewsSecondaryTab(array $values): static {
    $values += [
      'label' => $values['id'],
      'weight' => 1,
    ];

    $this->drupalGet('admin/structure/views_st');
    $this->getSession()->getPage()->clickLink('Add Views Secondary Tab');
    $this->submitForm($values, 'Save');

    return $this;
  }

  protected function installTheme(string $id): static {
    $this
      ->container
      ->get('theme_installer')
      ->install([$id]);

    return $this;
  }

  protected function setAdminTheme(string $id): static {
    $values = [
      'admin_theme' => $id,
      'use_admin_theme' => TRUE,
    ];
    $this->drupalGet('admin/appearance');
    $this->submitForm($values, 'Save configuration');

    return $this;
  }

}
